package org.insa.graphs.algorithm.shortestpath;

import org.insa.graphs.model.*;

import java.util.ArrayList;
import java.util.Collections;

import org.insa.graphs.algorithm.AbstractSolution.Status;
import org.insa.graphs.algorithm.utils.*;

public class DijkstraAlgorithm extends ShortestPathAlgorithm {

    public DijkstraAlgorithm(ShortestPathData data) {
        super(data);
    }

    protected void InitialiseLabels(Label[] labels, ShortestPathData data) {
        for (Node node: data.getGraph().getNodes()) {
        	labels[node.getId()] = new Label(node);
        }
    }
    
    @Override
    protected ShortestPathSolution doRun() {

        BinaryHeap<Label> tas = new BinaryHeap<Label>();
        
        final ShortestPathData data = getInputData();
        
        Graph graph = data.getGraph();
        Node origin = data.getOrigin();
        Node dest = data.getDestination();
        int nbNodes = graph.size();
        
        Label[] labels = new Label[nbNodes];
        //int nbNonMarque = nbNodes;
        this.InitialiseLabels(labels, data);
        
        labels[origin.getId()].actualiser(0.0, null);
        tas.insert(labels[origin.getId()]);
        notifyOriginProcessed(origin);
        
        // TEST SUR LES COUTS CROISSANTS
        int ancienID = origin.getId();
        
        boolean encore = true;
        
        while ((!labels[dest.getId()].isMarque()) && encore) {
        	
        	try {
            	Label current = tas.deleteMin();
            	
            	int currentID = current.getSommetCourant().getId();
            	labels[currentID].marquer();
            	notifyNodeMarked(current.getSommetCourant());
            	
            	// POUR LE TEST SUR LES COUTS CROISSANTS
            	if (labels[currentID].compareTo(labels[ancienID]) == -1) {
    	        	System.out.println("Erreur : Couts non croissants : noeud " + ancienID + " (" + labels[ancienID].getTotalCost() + "; " + labels[ancienID].getEstimation() + ") contre " + currentID + " (" + labels[currentID].getTotalCost() + "; " + labels[currentID].getEstimation() + ") ");
            	}
            	ancienID = currentID;
            	
            	//nbNonMarque--;
            	
            	// POUR LE TEST SUR LES SUCCESSEURS
            	int compteur_successeurs = 0;
            	for (Arc succ: labels[currentID].getSommetCourant().getSuccessors()) {
            		// POUR LE TEST SUR LES SUCCESSEURS
            		compteur_successeurs++;
            		if (!data.isAllowed(succ)) {
            			continue;
            		}
	            	int succID = succ.getDestination().getId();
	            	if (!labels[succID].isMarque()) {
	            		if (labels[succID].getCost() > (labels[currentID].getCost() + data.getCost(succ))) {
	            			if (labels[succID].getCost() < Double.POSITIVE_INFINITY) {
	            				tas.remove(labels[succID]);
	            			} else {
	            				notifyNodeReached(succ.getDestination());
	            			}
	            			labels[succID].actualiser(labels[currentID].getCost() + data.getCost(succ), succ);
	            			tas.insert(labels[succID]);
	            		}        			
	            	}
	            	
	            	// TEST SI LE TAS EST TOUJOURS VALIDE
	            	if (!tas.IsValid()) {
	            		System.out.println("ERREUR : Le tas n'est pas valide.");
	            	}
            	}
            	// POUR LE TEST SUR LES SUCCESSEURS
            	if (compteur_successeurs != labels[currentID].getSommetCourant().getNumberOfSuccessors()) {
            		System.out.println("ERREUR Nombre de successeurs visités : " + compteur_successeurs + "/" + labels[currentID].getSommetCourant().getNumberOfSuccessors());
            	}
            }
        	catch (EmptyPriorityQueueException e) {
        		encore = false;
        	}
        	catch (ElementNotFoundException e) {
        		encore = false;
        	}
        }
        
        ShortestPathSolution solution = null;
        //Pour le test SOMME(couts) = Cout(Label(Destination))
        double SommeCouts = 0.0;
        
     // Destination n'est pas marquée, the solution is infeasible...
        if (!labels[dest.getId()].isMarque()) {
            solution = new ShortestPathSolution(data, Status.INFEASIBLE);
        }
        else {

            // The destination has been found, notify the observers.
            notifyDestinationReached(dest);

            // Create the path from the array of predecessors...
            ArrayList<Arc> arcs = new ArrayList<>();
            Arc arc = labels[dest.getId()].getPere();
            
            while (arc != null) {
                arcs.add(arc);

                //Pour le test SOMME(couts) = Cout(Label(Destination))
                SommeCouts += data.getCost(arc);
                arc = labels[arc.getOrigin().getId()].getPere();
            }

            // Reverse the path...
            Collections.reverse(arcs);

            // Create the final solution.
            solution = new ShortestPathSolution(data, Status.OPTIMAL, new Path(graph, arcs));
        }
        
        if (solution.getStatus() != Status.INFEASIBLE) {
	        // Test si le path est valide
	        if (!solution.getPath().isValid()) {
	        	System.out.println("Erreur : PATH SOLUTION INVALID !");
	    	}
	        
	        // Test s'il correspond a Shortest et Fastest de la classe Path
	        // INUTILE, SUJET MAL COMPRIS
	        /*if (data.getMode() == Mode.LENGTH) {
	        	if (solution.getPath().getLength() != Path.createShortestPathFromNodes(graph, solution.getPath().getNodes()).getLength()) {
	            	System.out.println("Erreur : PATH SOLUTION NOT SHORTEST !");
	        	} else {
	        		System.out.println("PATH SOLUTION SHORTEST !");
	        	}
	        } else {
	        	if (solution.getPath().getMinimumTravelTime() != Path.createFastestPathFromNodes(graph, solution.getPath().getNodes()).getMinimumTravelTime()) {
	            	System.out.println("Erreur : PATH SOLUTION NOT FASTEST !");
	        	} else {
	        		System.out.println("PATH SOLUTION FASTEST !");
	        	}
	        }*/
	        
	        if ((int)(SommeCouts * 1000) != (int)(labels[dest.getId()].getCost() * 1000)) {
	        	System.out.println("Erreur : Cost label Dest (" + labels[dest.getId()].getCost() + ") != Somme(cost) (" + SommeCouts + ")");
	    	}
        }
                
        

        return solution;
    }

}
