package org.insa.graphs.algorithm.shortestpath;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;

import org.junit.BeforeClass;
import org.junit.Test;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;

import org.insa.graphs.model.io.BinaryGraphReader;
import org.insa.graphs.model.io.GraphReader;
import org.insa.graphs.model.Graph;

import org.insa.graphs.algorithm.shortestpath.BellmanFordAlgorithm;
import org.insa.graphs.algorithm.shortestpath.DijkstraAlgorithm;
import org.insa.graphs.algorithm.shortestpath.AStarAlgorithm;

import org.insa.graphs.algorithm.ArcInspectorFactory;
import org.insa.graphs.algorithm.shortestpath.ShortestPathData;

public class OptimaliteTest {

    // Graph utilisé pour les tests
    static Graph graph;
    static double delta = 0.00001;
    
    //Lecture du graphe
    @BeforeClass
    public static void initAll() throws Exception {
    	final String mapName = "/home/paulfaure/Documents/3A/JAVA/Cartes BE/haute-garonne/europe/france/haute-garonne.mapgr";

        // Create a graph reader.
        final GraphReader reader = new BinaryGraphReader(
                new DataInputStream(new BufferedInputStream(new FileInputStream(mapName))));

        // Read the graph.
        OptimaliteTest.graph = reader.read();
        
        reader.close();
    }

    @Test
    public void testCheminNul() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(12305), graph.get(12305), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertTrue(solutionDijkstra.isFeasible());
        assertTrue(solutionAStar.isFeasible());
        assertTrue(solutionDijkstra.getPath().isEmpty());
        assertTrue(solutionAStar.getPath().isEmpty());
    }
    
    /*@Test
    public void testOriginNotInGraph() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(-1), graph.get(12305), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertFalse(solutionDijkstra.isFeasible());
        assertFalse(solutionAStar.isFeasible());
    }
    
    @Test
    public void testDestinationNotInGraph() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(12305), graph.get(1230555), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertFalse(solutionDijkstra.isFeasible());
        assertFalse(solutionAStar.isFeasible());
    }*/
    
    @Test
    public void testAucunCheminPossible() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(137947), graph.get(75870), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertFalse(solutionDijkstra.isFeasible());
        assertFalse(solutionAStar.isFeasible());
    }
    
    @Test
    public void testOptimaliteLengthAllRoadAllowed() {
    	ShortestPathData data = new ShortestPathData(graph, graph.get(92173), graph.get(75870), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm bellmanFord = new BellmanFordAlgorithm(data);
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
    	ShortestPathSolution solutionBellmanFord = bellmanFord.run();
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionDijkstra.getPath().getLength(), OptimaliteTest.delta);
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionAStar.getPath().getLength(), OptimaliteTest.delta);
    }
    
    @Test
    public void testOptimaliteLengthOnlyRoadOpenForCarsAllowed() {
    	ShortestPathData data = new ShortestPathData(graph, graph.get(92173), graph.get(75870), ArcInspectorFactory.getAllFilters().get(1));
    	ShortestPathAlgorithm bellmanFord = new BellmanFordAlgorithm(data);
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
    	ShortestPathSolution solutionBellmanFord = bellmanFord.run();
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionDijkstra.getPath().getLength(), OptimaliteTest.delta);
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionAStar.getPath().getLength(), OptimaliteTest.delta);
    }
    
    @Test
    public void testOptimaliteTimeOnlyRoadOpenForCarsAllowed() {
    	ShortestPathData data = new ShortestPathData(graph, graph.get(92173), graph.get(75870), ArcInspectorFactory.getAllFilters().get(2));
    	ShortestPathAlgorithm bellmanFord = new BellmanFordAlgorithm(data);
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
    	ShortestPathSolution solutionBellmanFord = bellmanFord.run();
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionDijkstra.getPath().getLength(), OptimaliteTest.delta);
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionAStar.getPath().getLength(), OptimaliteTest.delta);
    }
    
    @Test
    public void testOptimaliteTimePedestriansAndBicyclesAllowed() {
    	ShortestPathData data = new ShortestPathData(graph, graph.get(92173), graph.get(75870), ArcInspectorFactory.getAllFilters().get(3));
    	ShortestPathAlgorithm bellmanFord = new BellmanFordAlgorithm(data);
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
    	ShortestPathSolution solutionBellmanFord = bellmanFord.run();
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionDijkstra.getPath().getLength(), OptimaliteTest.delta);
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionAStar.getPath().getLength(), OptimaliteTest.delta);
    }
    
    @Test
    public void testErreurEstimationLengthAllRoadAllowed() {
    	ShortestPathData data = new ShortestPathData(graph, graph.get(7710), graph.get(6299), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm bellmanFord = new BellmanFordAlgorithm(data);
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
    	ShortestPathSolution solutionBellmanFord = bellmanFord.run();
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionDijkstra.getPath().getLength(), OptimaliteTest.delta);
        assertEquals(solutionBellmanFord.getPath().getLength(),solutionAStar.getPath().getLength(), OptimaliteTest.delta);
    }
}
