package org.insa.graphs.algorithm.shortestpath;

import static org.junit.Assert.assertTrue;

import org.junit.BeforeClass;
import org.junit.Test;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.FileInputStream;

import org.insa.graphs.model.io.BinaryGraphReader;
import org.insa.graphs.model.io.GraphReader;
import org.insa.graphs.model.Graph;

import org.insa.graphs.algorithm.shortestpath.DijkstraAlgorithm;
import org.insa.graphs.algorithm.shortestpath.AStarAlgorithm;

import org.insa.graphs.algorithm.ArcInspectorFactory;
import org.insa.graphs.algorithm.shortestpath.ShortestPathData;

public class PerformancesTest {

    // Graph utilisé pour les tests
    static Graph graph;
    
    //Lecture du graphe
    @BeforeClass
    public static void initAll() throws Exception {
    	final String mapName = "/home/paulfaure/Documents/3A/JAVA/Cartes BE/haute-garonne/europe/france/haute-garonne.mapgr";

        // Create a graph reader.
        final GraphReader reader = new BinaryGraphReader(
                new DataInputStream(new BufferedInputStream(new FileInputStream(mapName))));

        // Read the graph.
        PerformancesTest.graph = reader.read();

        reader.close();
    }
    
    @Test
    public void testDijkstraPlusRapidePetitTrajet() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(87500), graph.get(103867), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertTrue(solutionDijkstra.getSolvingTime().compareTo(solutionAStar.getSolvingTime()) < 0);
    }
    
    @Test
    public void testDijkstraPlusRapideCheminNul() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(103867), graph.get(103867), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertTrue(solutionDijkstra.getSolvingTime().compareTo(solutionAStar.getSolvingTime()) < 0);
    }
    
    @Test
    public void testDijkstraPlusRapideCheminInexistant() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(135847), graph.get(103867), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertTrue(solutionDijkstra.getSolvingTime().compareTo(solutionAStar.getSolvingTime()) < 0);
    }
    
    @Test
    public void testAStarPlusRapideLongTrajet() {
    	// On ne fait le test qu'avec un filtre sur les chemins autorisés
    	ShortestPathData data = new ShortestPathData(graph, graph.get(12305), graph.get(103867), ArcInspectorFactory.getAllFilters().get(0));
    	ShortestPathAlgorithm dijkstra = new DijkstraAlgorithm(data);
    	ShortestPathAlgorithm aStar = new AStarAlgorithm(data);
    	ShortestPathSolution solutionAStar = aStar.run();
    	ShortestPathSolution solutionDijkstra = dijkstra.run();
        assertTrue(solutionDijkstra.getSolvingTime().compareTo(solutionAStar.getSolvingTime()) > 0);
    }
}
